@@ -22,6 +22,7 @@
 #include <sys/mount.h>
 #include <sys/ioctl.h>
 
+int platform_has_uuid = 1;
 extern char *progname;
 
 int
@@ -91,10 +92,10 @@ platform_check_iswritable(char *name, char *block, struct stat64 *s, int fatal)
 	return 0;
 }
 
-void
-platform_set_blocksize(int fd, char *path, dev_t device, int blocksize)
+int
+platform_set_blocksize(int fd, char *path, dev_t device, int blocksize, int fatal)
 {
-	return;
+	return fatal;
 }
 
 void
@@ -178,3 +179,9 @@ platform_align_blockdev(void)
 {
 	return (sizeof(void *));
 }
+
+int
+platform_nproc(void)
+{
+	return 1;
+}
